'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

const NOT_MOUNTED = null;
const INIT = 'INIT';
const SUCCESS = 'SUCCESS';
const FAIL = 'FAIL';
const useAsync = ({
  asyncFunction,
  params,
  onInit,
  onSuccess,
  onFail
}) => {
  const promiseRef = react.useRef(null);
  const canceled = react.useRef(false);
  const [state, setState] = react.useState(NOT_MOUNTED);
  const invoke = react.useCallback((...args) => {
    if (onInit) {
      onInit({
        result: null,
        error: null,
        args
      });
    }

    setState(INIT);
    const promiseOrResult = asyncFunction(...args);

    if (promiseOrResult instanceof Promise) {
      promiseRef.current = promiseOrResult;
      promiseOrResult.then(result => {
        setState(SUCCESS);

        if (!canceled.current) {
          if (onSuccess) {
            onSuccess({
              result,
              error: null,
              args
            });
          }
        }

        return result;
      }).catch(error => {
        setState(FAIL);

        if (!canceled.current) {
          if (onFail) {
            onFail({
              result: null,
              error,
              args
            });
          }
        }
      });
    } else {
      setState(SUCCESS);

      if (onSuccess) {
        onSuccess({
          result: promiseOrResult,
          error: null,
          args
        });
      }

      Promise.resolve(promiseOrResult).catch(() => {});
    }
  }, [asyncFunction, onInit, onFail, onSuccess]);
  react.useEffect(() => {
    canceled.current = false;
    return () => {
      setState(NOT_MOUNTED);
      canceled.current = true;
    };
  }, []);
  react.useEffect(() => {
    if (params !== undefined) {
      invoke(...params);
    }
  }, [invoke, params]);
  return {
    state,
    invoke,
    promise: promiseRef.current
  };
};

const useAsyncWithDispatch = ({
  asyncFunction,
  params,
  dispatch,
  actions
}) => {
  const {
    INIT,
    SUCCESS,
    FAILURE
  } = actions;
  const onSuccess = react.useCallback(({
    result
  }) => {
    dispatch({
      type: SUCCESS,
      payload: result
    });
  }, [dispatch, SUCCESS]);
  const onInit = react.useCallback(() => {
    dispatch({
      type: INIT,
      payload: null
    });
  }, [dispatch, INIT]);
  const onFail = react.useCallback(({
    error
  }) => {
    dispatch({
      type: FAILURE,
      payload: error
    });
  }, [dispatch, FAILURE]);
  return useAsync({
    asyncFunction,
    params,
    onFail,
    onInit,
    onSuccess
  });
};

const defaultState = {
  isProcessing: false,
  error: null,
  result: null
};

const useAsyncState = initialState => {
  const [value, setValue] = react.useState({ ...defaultState,
    ...(initialState || {})
  });
  const setResult = react.useCallback(result => setValue({ ...defaultState,
    result
  }), []);
  const setLoading = react.useCallback(() => {
    setValue({ ...defaultState,
      isProcessing: true
    });
  }, []);
  const setFailed = react.useCallback(error => setValue({ ...defaultState,
    error
  }), []);
  return {
    value,
    setLoading,
    setFailed,
    setResult
  };
};

const useAsyncWithState = props => {
  const {
    value,
    setFailed,
    setLoading,
    setResult
  } = useAsyncState(props.initialState);
  const onSuccess = react.useCallback(({
    result
  }) => {
    setResult(result);
  }, [setResult]);
  const onInit = react.useCallback(() => {
    setLoading();
  }, [setLoading]);
  const onFail = react.useCallback(({
    error
  }) => {
    setFailed(error);
  }, [setFailed]);
  const returnProps = useAsync({ ...props,
    onSuccess,
    onInit,
    onFail
  });
  return { ...returnProps,
    ...value
  };
};

exports.FAIL = FAIL;
exports.INIT = INIT;
exports.NOT_MOUNTED = NOT_MOUNTED;
exports.SUCCESS = SUCCESS;
exports.useAsync = useAsync;
exports.useAsyncWithDispatch = useAsyncWithDispatch;
exports.useAsyncWithState = useAsyncWithState;
